// Adafruit IO RGB LED Output Example
//
// Adafruit invests time and resources providing this open source code.
// Please support Adafruit and open source hardware by purchasing
// products from Adafruit!
//
// Written by Todd Treece for Adafruit Industries
// Copyright (c) 2016-2017 Adafruit Industries
// Licensed under the MIT license.
//
// All text above must be included in any redistribution.

/************************** Configuration ***********************************/

// edit the config.h tab and enter your Adafruit IO credentials
// and any additional configuration needed for WiFi, cellular,
// or ethernet clients.
#include "config.h"

/************************ Example Starts Here *******************************/

#include "Adafruit_NeoPixel.h"

#define PIXEL_PIN     14
#define PIXEL_COUNT   16
#define PIXEL_TYPE    NEO_GRB + NEO_KHZ800

int animationState = 0; //0: Solid, 1: Chase, 2: Twinkle
int colorState = 1; //0: Solid Color, 1: Rainbow, 2: Color Cycle

Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, PIXEL_TYPE);

// set up the 'color' feed
AdafruitIO_Feed *color = io.feed("trophio-color-picker");
AdafruitIO_Feed *rainbow_button = io.feed("trophio-rainbow-mode");
AdafruitIO_Feed *color_cycle = io.feed("trophio-color-cycle");
AdafruitIO_Feed *twinkle_mode = io.feed("trophio-twinkle-mode");
AdafruitIO_Feed *pulse_mode = io.feed("trophio-pulse-mode");

void setup() {

  // start the serial connection
  Serial.begin(115200);

  // wait for serial monitor to open
  while (! Serial);

  // connect to io.adafruit.com
  Serial.print("Connecting to Adafruit IO");
  io.connect();
  
  // neopixel init
  strip.begin();
  strip.show();


  // set up a message handler for the 'color' feed.
  // the handleMessage function (defined below)
  // will be called whenever a message is
  // received from adafruit io.
  color->onMessage(handleColor);
  rainbow_button->onMessage(handleRainbow);
  color_cycle->onMessage(handleColorCycle);
  twinkle_mode->onMessage(handleTwinkleMode);
  pulse_mode->onMessage(handlePulseMode);  

  // wait for a connection
  while (io.status() < AIO_CONNECTED) {
    strip.setPixelColor(random(0,15), strip.Color(random(0,255), random(0,255), random(0,255)));
    strip.show();
    delay(500);
  }

  // we are connected
  Serial.println();
  Serial.println(io.statusText());

}

void loop() {

  // io.run(); is required for all sketches.
  // it should always be present at the top of your loop
  // function. it keeps the client connected to
  // io.adafruit.com, and processes any incoming data.
  io.run();

  switch (colorState) {
    case 1:
      // Rainbow
      rainbow(20);
      break;
    case 2:
      //color cycle
      colorWipe(strip.Color(255, 0, 0), 50); // Red
      colorWipe(strip.Color(0, 255, 0), 50); // Green
      colorWipe(strip.Color(0, 0, 255), 50); // Blue
      break;
     case 3: 
      twinkle(100);
      break;
     case 4:
      pulse();
      break;
  }

}

// this function is called whenever a 'color' message
// is received from Adafruit IO. it was attached to
// the color feed in the setup() function above.
void handleColor(AdafruitIO_Data *data) {
  updateColorState(0);
  // print RGB values and hex value
  Serial.println("Received HEX: ");
  Serial.println(data->value());

  long color = data->toNeoPixel();

  for (int i = 0; i < PIXEL_COUNT; ++i) {
    strip.setPixelColor(i, color);
  }

  strip.show();

}
void handleRainbow(AdafruitIO_Data *data) {
  updateColorState(1);
}
void handleColorCycle(AdafruitIO_Data *data) {

  updateColorState(2);

}
void handleTwinkleMode(AdafruitIO_Data *data) {

  updateColorState(3);

}
void handlePulseMode(AdafruitIO_Data *data) {

  updateColorState(4);

}
void updateColorState(int state) {
  colorState = state;
  Serial.println();
  Serial.println(colorState);
}
void pulse(){
  for(int i = 0; i < 255; i++){
    for(int j=0; j<16; j++){
      strip.setPixelColor(j, strip.Color(i, i, i));
    }
    strip.show();
    delay(25);
  }
  for(int i = 255; i > 0; i--){
    for(int j=0; j<16; j++){
      strip.setPixelColor(j, strip.Color(i, i, i));
    }
    strip.show();
    delay(25);
  }
}
void twinkle(int wait){
  for(int i = 0; i < 16; i++){
    int randNum = random(0, 255);
    strip.setPixelColor(i, strip.Color(randNum, randNum, randNum));
    strip.show();
    delay(wait);
    strip.setPixelColor(i, 0);
    strip.show();
  }
}

void colorWipe(uint32_t c, uint8_t wait) {
  for (uint16_t i = 0; i < strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for (j = 0; j < 256; j++) {
    for (i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for (j = 0; j < 256 * 5; j++) { // 5 cycles of all colors on wheel
    for (i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
  for (int j = 0; j < 10; j++) { //do 10 cycles of chasing
    for (int q = 0; q < 3; q++) {
      for (uint16_t i = 0; i < strip.numPixels(); i = i + 3) {
        strip.setPixelColor(i + q, c);  //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (uint16_t i = 0; i < strip.numPixels(); i = i + 3) {
        strip.setPixelColor(i + q, 0);      //turn every third pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j = 0; j < 256; j++) {   // cycle all 256 colors in the wheel
    for (int q = 0; q < 3; q++) {
      for (uint16_t i = 0; i < strip.numPixels(); i = i + 3) {
        strip.setPixelColor(i + q, Wheel( (i + j) % 255)); //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (uint16_t i = 0; i < strip.numPixels(); i = i + 3) {
        strip.setPixelColor(i + q, 0);      //turn every third pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if (WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
